/*
 * Copyright (C) 2010-2020 Alibaba Group Holding Limited
 */
#ifndef NATIVE_RECORDER_H
#define NATIVE_RECORDER_H

#include <stdint.h>
#include "native_recorder_callback.h"
#include "render_engine/render_engine.h"
#include "common/svideo_export.h"
#include "layout/inc/layout_def.h"
#include <memory>


namespace avcore
{
class MicroCoreHandler;
}

namespace AlivcConan
{
class AlivcEventReport;
};


NS_SVIDEO_BEGIN

class AlivcLicense;
struct AlivcLicenseParam;
class IRecorderService;
class RecorderAudioSourceModel;

enum RecordType
{
    SINGLE_RECORDER = 0,
    MULTI_RECORDER = 1
};

class SVIDEO_EXPORT NativeRecorder
{
public:
    NativeRecorder();

    ~NativeRecorder();


public:
    //------------------生命周期------------------
    /**
     * 初始化Recorder
     * @param reportId 埋点数据的reportId
     * @param logId debug日志上传的Id
     * @return 成功：ALIVC_COMMON_RETURN_SUCCESS
     */
    int Prepare(int64_t reportId, int64_t logId, int recordType = SINGLE_RECORDER);

    /**
     * 释放Recorder资源
     */
    void Release();

    /**
     * 释放Recorder资源
     */
    int CreateTextureId();

    //----------------录制编码参数-----------------

    /**
     * @brief SetParam 设置编码参数
     * @param key  key值
     * @param value 取值
     */
    void SetParam(int key, int value);

    /**
     * @brief QuietAudioStream 静音录制，输出视频仍然含有音频流，只是音频流内容为静音内容
     * @param mute 是否静音录制
     */
    void QuietAudioStream(bool mute);

    /**
     * @brief SetRecVideoSize 设置录制视频的分辨率
     * @param width 宽
     * @param height 高
     */
    void SetRecVideoSize(int width, int height);
    //------------------录制控制------------------
    /**
     * 开始录制一个片段
     * @param path 输出文件路径
     * @return 成功：ALIVC_COMMON_RETURN_SUCCESS
     */
    int Start(const char* path, int64_t maxDuration = -1);

    /**
     * 停止录制该片段，生成视频
     * @return 成功：ALIVC_COMMON_RETURN_SUCCESS
     */
    int Stop();

    /**
     * 取消录制该片段，无效视频，缓存不会删除
     */
    void Cancel();

    /**
     * 将一些片段拼接为一个视频文件
     * @param input 输入文件列表
     * @param count 输入文件个数
     * @param output 输出文件地址
     * @return 成功：ALIVC_COMMON_RETURN_SUCCESS
     */
    int StitchPart(const char ** input, int count, const char* output);

    //----------------录制渲染相关-----------------

    /**
     * 标记开始预览；直接SetDisplay也会进入Preview状态
     */
    int StartPreview();

    /**
     * 设置预览输出view（同步）
     * @param display 显示view的指针
     * 正常返回 ALIVC_COMMON_RETURN_SUCCESS
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     * 设置预览输出失败 ALIVC_SVIDEO_EDITOR_SET_DISPLAY_FAILED
     */
    int SetDisplay(void *display);

    /**
     * @brief SetDisplaySize 设置浏览视频的分辨率,上层surfaceView布局变化时使用
     * @param width 宽
     * @param height 高
     */
    int SetDisplaySize(int width, int height);

    /**
     设置渲染矩阵

     @param mMatrix Matrix
     */
    void SetSurfaceMatrix(float* mMatrix, int width, int height);

    /**
     * @brief SetCaptureSize 设置采集的纹理宽高，android需要使用
     * @param surface surfaceTexture的global ref
     * @param width
     * @param height
     */
    int SetCaptureSurface(void* obj, int width, int height, int rotate);

    /**
     * @brief SetPureColorBorder 设置纯色边框
     * @param layoutId 目标区域ID
     * @param borderWidth 相对值（相对于输出宽度）
     * @param borderColor 颜色值argb
     * @param cornerRadius 相对值（相对于输出宽度）
     */
    int SetPureColorBorder(int layoutId, float borderWidth, uint32_t borderColor, float cornerRadius);

    /**
     * @brief RemovePureColorBorder 移除纯色边框
     * @param layoutId 目标区域ID
     */
    void RemovePureColorBorder(int layoutId);

    /**
     * @brief SetBeautyLevel 设置美颜程度
     * @param level 取值[0， 100],越大美颜效果越强
     */
    void SetBeautyLevel(int level);

    /**
     * 使用滤镜（同步）
     * @param filePath 文件路径
     * @return
     * 正常返回 ALIVC_COMMON_RETURN_SUCCESS
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     */
    int ApplyFilter(const char *filePath);

    /**
     * 删除滤镜（同步）
     * @return
     * 正常返回 ALIVC_COMMON_RETURN_SUCCESS
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     */
    int RemoveFilter();

    /**
     * 使用特效滤镜（同步）
     * @param filePath 文件路径
     * @return
     * 正常返回 资源id
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     * 文件不存在 ALIVC_SVIDEO_EDITOR_FILE_NOT_EXIST
     */
    int ApplyAnimationFilter(const char *filePath, const char *paramsString);

    /**
    * 更新特效滤镜（同步）
    * @param vid vid
    * @param paramsString 参数string
    * @return
    */
    int UpdateAnimationFilter(int vid, const char *paramsString);

    /**
     * 删除特效滤镜（同步）
     * @return
     * 正常返回 资源id
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     * 文件不存在 ALIVC_SVIDEO_EDITOR_FILE_NOT_EXIST
     */
    int RemoveAnimationFilter();
    /**
     * 添加动图
     * @param filePath
     * @param id 唯一标示
     * @param x 相对值 [0 - 1]
     * @param y 相对值 [0 - 1]
     * @param width 相对值 [0 - 1]
     * @param height 相对值 [0 - 1]
     * @param rotate 旋转角度
     * @param flip 是否翻转
     * @param duration 起始时间为当前录制时间，duration代表播放时长
     * @param faceType 人脸动图1， 普通动图0
     * @return
     * 正常返回 资源id
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     * 文件不存在 ALIVC_SVIDEO_EDITOR_FILE_NOT_EXIST
     * 解析动图失败 ALIVC_SVIDEO_EDITOR_PARSE_RESOURCE_FAILED
     */
    int AddGifView(const char *filePath, int id, float x, float y, float width, float height,
                   float rotate, bool flip, int64_t duration, int faceType);

    /**
     * 添加静态图片，通过路径的模式（同步）
     * @param layoutId 目标区域ID
     * @param filePath
     * @param id 唯一标示
     * @param x 相对值 [0 - 1]
     * @param y 相对值 [0 - 1]
     * @param width 相对值 [0 - 1]
     * @param height 相对值 [0 - 1]
     * @param rotate 旋转角度
     * @return
     * 正常返回 资源id
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     * 文件不存在 ALIVC_SVIDEO_EDITOR_FILE_NOT_EXIST
     * 图片类型不支持 ALIVC_SVIDEO_EDITOR_VIEW_TYPE_NOT_SUPPORTED
     */
    int AddImageView(int layoutId, const char *filePath, int id, float x, float y, float width, float height,
                     float rotate);

    /**
     * 添加静态图片，通过数据的模式（同步）
     * @param layoutId 目标区域ID
     * @param data 图片数据
     * @param id 唯一标示
     * @param dataWidth 图片数据宽
     * @param dataHeight 图片数据高
     * @param widthStep 数据步长
     * @param x 相对值 [0 - 1]
     * @param y 相对值 [0 - 1]
     * @param width 相对值 [0 - 1]
     * @param height 相对值 [0 - 1]
     * @param rotate 旋转角度
     * @return
     * 正常返回 资源id
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     */
    int AddImageView(int layoutId, const uint8_t* data, int id, int dataWidth, int dataHeight, int widthStep,
                     float x, float y, float width, float height, float rotate);


    /**
     * 删除vid指向的视图（同步）
     * @param layoutId 目标区域ID
     * @param vid 资源id
     * @return
     * 正常返回 ALIVC_COMMON_RETURN_SUCCESS
     * 状态不正确 ALIVC_COMMON_INVALID_STATE
     */
    int DeleteView(int layoutId, int id);

    /**
     * @brief setFace
     * @param vid 设置人来呢动图的view id
     * @param face face人脸坐标
     * @param size 坐标点个数
     */
    void SetFace(int vid, float* face, int size);

    /**
     * @brief FrameAvailable 采集数据准备就绪
     * @param tid 纹理id
     * @param timestamp 系统时间戳
     * @param isLast    是否为多采集最后一帧
     * @param layoutId  采集区域id
     *
     */
    int FrameAvailable(int tid, int64_t timestamp, bool isLast, int layoutId);

    /**
     * @brief FrameAvailable 采集数据准备就绪
     * @param tid 纹理id
     * @param timestamp 系统时间戳
     * @param isLast    是否为多采集最后一帧
     * @param layoutId  采集区域id
     */
    int FrameAvailable(void *sampleBuffer, int64_t timestamp, bool isLast, int layoutId);

    /**
     * @brief TakePhoto 拍照，通过NativeScreenshotCallback 回调数据
     */
    int TakePhoto();

    /**
     * @brief setViewPosition 设置view的位置和尺寸
     * @param layoutId 目标区域ID
     * @param vid 设置的目标view的id
     * @param x 起始横坐标起点
     * @param y 起始纵坐标起点
     * @param width 设置后view的宽
     * @param height 设置后view的高
     */
    int UpdateViewPosition(int layoutId, int vid, float x, float y, float width, float height);

    /**
     * @brief MapScreenToOriginalPreview 获取渲染坐标映射后的值
     * @param point 转换后的值
     */
    void MapScreenToOriginalPreview(float* point);


    /**
     * @brief SetVideoFlip 设置输出视频进行镜像，浏览不镜像
     * @param flip true打开，false关闭
     */
    int SetVideoFlip(int layoutId, bool flip);

    int AddCaptureLayout(int layoutLevel, float* layoutParam, bool mainTrack, NS_SVIDEO::ResolutionPolicy policy);

    int UpdateCaptureLayout(int layoutId, int layoutLevel, float* layoutParam);

    int RemoveCaptureLayout(int layoutId);

    //----------------录制音频相关-----------------

    /**
     * @brief AddBgMusic 添加背景音乐，录音AddSourceData将失效，默认全局添加音乐且自动循环播放
     * @param input  输入文件的绝对路径
     * @param startTime 相对于文件的开始时间，单位微秒，用于选取音乐片段
     * @param duration 选取片段的长度，单位微秒
     * @param maxRecordDuration 录制的最大时长
     * @return 成功：ALIVC_COMMON_RETURN_SUCCESS
     */
    int AddBgMusic(const char* input, int64_t startTime, int64_t duration, int64_t maxRecordDuration);

    /**
     * @brief AddBgMusic 设置背景音乐seek，当配乐模式下，回删了片段，再次开始时背景音乐需要回退
     * @param seekTime  输入的是所有录制出来视频的长度即clipManager的duration，
        内部会根据视频长度计算出对应的需要seek的音频文件时间点，
     * @return 成功：ALIVC_COMMON_RETURN_SUCCESS
     */
    int SetAudioSeek(int64_t seekTime);
    int AddAudioSource(RecorderAudioSourceModel *source, bool isMic);
    int UpdateAudioSource(RecorderAudioSourceModel *source);
    int RemoveAudioSource(int audioId);
    int SetAECConfig(int nearAudioId, int farAudioId, bool needMixFar);
    int RemoveAECConfig();

    /**
     * @brief AddSourceData 添加采集的音频数据，使用bgMusic之后该接口会失效
     * @param data  添加的数据指针
     * @param length 输出数据的长度
     * @param sample 输入数据的sample个数
     * @param timestamp 时间戳，采用系统时间。
     */
    int AddSourceData(uint8_t* data, int length, int sample, int64_t timestamp);
    int AddSourceData(uint8_t* data, int length, int sample, int64_t timestamp, int audioId);

    //-----------------时间特效-------------------
    /**
     * @brief SetTempo 设置输出视频的速度，此时音频流数据变速不变调,对于音乐或者采集音频生效，视频流速度也对应修改
     * @param factor 变速比例取值范围[0.5, 2]
     * @return 成功：ALIVC_COMMON_RETURN_SUCCESS
     */
    int SetVideoTempo(float factor);
    /**
     * @brief 强制渲染引擎执行onRenderDestroy回调
     */
    void ForceRenderDestroyCallback();


    //-------------------其他---------------------
    /**
     * @brief SetCallback 设置回调，回调调用在RecService的线程
     * @param cb 回调的实现
     */
    void SetCallback(const std::shared_ptr<NativeRecCallback>& cb);

    /**
     * @brief setTextureCallback 设置当前纹理的回调，回调调用在RecService的线程
     * @param cb 回调的实现
     */
    void SetTextureCallback(const std::shared_ptr<NativeRecTextureCallback>& cb);

    /**
     * @brief SetScreenshotCallback 设置拍照回调，回调调用在RecService的线程
     * @param cb 回调的实现
     */
    void SetScreenshotCallback(const std::shared_ptr<NativeScreenshotCallback>& cb);

    bool CheckLicenseInvalid();


    /**
     * 设置背景颜色
     * @param layoutId
     * @param color
     * @return
     */
    int SetBackgroundColor(int layoutId, long color);

    /**
     * 设置背景图片填充模式
     * @param layoutId
     * @param displayMode
     * @return
     */
    int SetBackgroundDisplayMode(int layoutId, int displayMode);

    /**
     * 设置背景图片
     * @param layoutId
     * @param data
     * @param dataWidth
     * @param dataHeight
     * @param widthStep
     * @return
     */
    int SetBackgroundBuffer(int layoutId, void *data, int dataWidth, int dataHeight, int widthStep);

    /**
     * 清空背景
     * @param layoutId
     * @return
     */
    int ClearBackground(int layoutId);

private:
    std::shared_ptr<IRecorderService> mService {nullptr};
    std::shared_ptr<MicroCoreHandler> mCoreHandler{nullptr};

    AlivcConan::AlivcEventReport*   mEventReport{nullptr};
    int                             mRecordType;
};

NS_SVIDEO_END

#endif //NATIVE_RECORDER_H
